from multidict import istr

ACCEPT = istr("Accept")
ACCEPT_CHARSET = istr("Accept-Charset")
ACCEPT_ENCODING = istr("Accept-Encoding")
ACCEPT_LANGUAGE = istr("Accept-Language")
ACCEPT_RANGES = istr("Accept-Ranges")
ACCESS_CONTROL_ALLOW_CREDENTIALS = istr("Access-Control-Allow-Credentials")
ACCESS_CONTROL_ALLOW_HEADERS = istr("Access-Control-Allow-Headers")
ACCESS_CONTROL_ALLOW_METHODS = istr("Access-Control-Allow-Methods")
ACCESS_CONTROL_ALLOW_ORIGIN = istr("Access-Control-Allow-Origin")
ACCESS_CONTROL_EXPOSE_HEADERS = istr("Access-Control-Expose-Headers")
ACCESS_CONTROL_MAX_AGE = istr("Access-Control-Max-Age")
ACCESS_CONTROL_REQUEST_HEADERS = istr("Access-Control-Request-Headers")
ACCESS_CONTROL_REQUEST_METHOD = istr("Access-Control-Request-Method")
AGE = istr("Age")
ALLOW = istr("Allow")
ALT_SVC = istr("Alt-Svc")
AUTHORIZATION = istr("Authorization")
CACHE_CONTROL = istr("Cache-Control")
CLEAR_SITE_DATA = istr("Clear-Site-Data")
CONNECTION = istr("Connection")
CONTENT_DISPOSITION = istr("Content-Disposition")
CONTENT_ENCODING = istr("Content-Encoding")
CONTENT_LANGUAGE = istr("Content-Language")
CONTENT_LENGTH = istr("Content-Length")
CONTENT_LOCATION = istr("Content-Location")
CONTENT_RANGE = istr("Content-Range")
CONTENT_SECURITY_POLICY = istr("Content-Security-Policy")
CONTENT_SECURITY_POLICY_REPORT_ONLY = istr("Content-Security-Policy-Report-Only")
CONTENT_TYPE = istr("Content-Type")
COOKIE = istr("Cookie")
COOKIE2 = istr("Cookie2")
DNT = istr("DNT")
DATE = istr("Date")
ETAG = istr("ETag")
EARLY_DATA = istr("Early-Data")
EXPECT = istr("Expect")
EXPECT_CT = istr("Expect-CT")
EXPIRES = istr("Expires")
FEATURE_POLICY = istr("Feature-Policy")
FORWARDED = istr("Forwarded")
FROM = istr("From")
HOST = istr("Host")
IF_MATCH = istr("If-Match")
IF_MODIFIED_SINCE = istr("If-Modified-Since")
IF_NONE_MATCH = istr("If-None-Match")
IF_RANGE = istr("If-Range")
IF_UNMODIFIED_SINCE = istr("If-Unmodified-Since")
INDEX = istr("Index")
KEEP_ALIVE = istr("Keep-Alive")
LARGE_ALLOCATION = istr("Large-Allocation")
LAST_MODIFIED = istr("Last-Modified")
LOCATION = istr("Location")
ORIGIN = istr("Origin")
PRAGMA = istr("Pragma")
PROXY_AUTHENTICATE = istr("Proxy-Authenticate")
PROXY_AUTHORIZATION = istr("Proxy-Authorization")
PUBLIC_KEY_PINS = istr("Public-Key-Pins")
PUBLIC_KEY_PINS_REPORT_ONLY = istr("Public-Key-Pins-Report-Only")
RANGE = istr("Range")
REFERER = istr("Referer")
REFERRER_POLICY = istr("Referrer-Policy")
RETRY_AFTER = istr("Retry-After")
SEC_WEBSOCKET_ACCEPT = istr("Sec-WebSocket-Accept")
SERVER = istr("Server")
SERVER_TIMING = istr("Server-Timing")
SET_COOKIE = istr("Set-Cookie")
SET_COOKIE2 = istr("Set-Cookie2")
SOURCEMAP = istr("SourceMap")
STRICT_TRANSPORT_SECURITY = istr("Strict-Transport-Security")
TE = istr("TE")
TIMING_ALLOW_ORIGIN = istr("Timing-Allow-Origin")
TK = istr("Tk")
TRAILER = istr("Trailer")
TRANSFER_ENCODING = istr("Transfer-Encoding")
UPGRADE_INSECURE_REQUESTS = istr("Upgrade-Insecure-Requests")
USER_AGENT = istr("User-Agent")
VARY = istr("Vary")
VIA = istr("Via")
WWW_AUTHENTICATE = istr("WWW-Authenticate")
WARNING = istr("Warning")
X_CONTENT_TYPE_OPTIONS = istr("X-Content-Type-Options")
X_DNS_PREFETCH_CONTROL = istr("X-DNS-Prefetch-Control")
X_FORWARDED_FOR = istr("X-Forwarded-For")
X_FORWARDED_HOST = istr("X-Forwarded-Host")
X_FORWARDED_PROTO = istr("X-Forwarded-Proto")
X_FRAME_OPTIONS = istr("X-Frame-Options")
X_XSS_PROTECTION = istr("X-XSS-Protection")
